@@ -1,7 +1,7 @@
+import datetime
 from tests.test_helper import *
 from braintree.test.credit_card_defaults import CreditCardDefaults
 from braintree.test.credit_card_numbers import CreditCardNumbers
-import braintree.test.venmo_sdk as venmo_sdk
 
 class TestCreditCard(unittest.TestCase):
     def test_create_with_three_d_secure_nonce(self):
@@ -18,15 +18,52 @@ class TestCreditCard(unittest.TestCase):
 
         three_d_secure_info = result.credit_card.verification.three_d_secure_info
 
-        self.assertEqual("Y", three_d_secure_info.enrolled)
         self.assertEqual("authenticate_successful", three_d_secure_info.status)
         self.assertEqual(True, three_d_secure_info.liability_shifted)
         self.assertEqual(True, three_d_secure_info.liability_shift_possible)
-        self.assertEqual("cavv_value", three_d_secure_info.cavv)
-        self.assertEqual("xid_value", three_d_secure_info.xid)
-        self.assertEqual(None, three_d_secure_info.ds_transaction_id)
-        self.assertEqual("05", three_d_secure_info.eci_flag)
-        self.assertEqual("1.0.2", three_d_secure_info.three_d_secure_version)
+        self.assertIsInstance(three_d_secure_info.enrolled, str)
+        self.assertIsInstance(three_d_secure_info.cavv, str)
+        self.assertIsInstance(three_d_secure_info.xid, str)
+        self.assertIsInstance(three_d_secure_info.eci_flag, str)
+        self.assertIsInstance(three_d_secure_info.three_d_secure_version, str)
+
+    def test_create_with_three_d_secure_pass_thru(self):
+        customer_id = Customer.create().customer.id
+        result = CreditCard.create({
+            "customer_id": customer_id,
+            "number": "4111111111111111",
+            "expiration_date": "05/2009",
+            "three_d_secure_pass_thru": {
+                "three_d_secure_version": "1.1.1",
+                "eci_flag": "05",
+                "cavv": "some-cavv",
+                "xid": "some-xid"
+            },
+            "options": {
+                "verify_card": "true",
+            }
+        })
+
+        self.assertTrue(result.is_success)
+
+    def test_create_with_three_d_secure_pass_thru_without_eci_flag(self):
+        customer_id = Customer.create().customer.id
+        result = CreditCard.create({
+            "customer_id": customer_id,
+            "number": "4111111111111111",
+            "expiration_date": "05/2009",
+            "three_d_secure_pass_thru": {
+                "three_d_secure_version": "1.1.1",
+                "cavv": "some-cavv",
+                "xid": "some-xid"
+            },
+            "options": {
+                "verify_card": "true",
+            }
+        })
+
+        self.assertFalse(result.is_success)
+        self.assertEqual("EciFlag is required.", result.message)
 
     def test_create_adds_credit_card_to_existing_customer(self):
         customer = Customer.create().customer
@@ -48,7 +85,6 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual("05/2014", credit_card.expiration_date)
         self.assertEqual("John Doe", credit_card.cardholder_name)
         self.assertNotEqual(re.search(r"\A\w{32}\Z", credit_card.unique_number_identifier), None)
-        self.assertFalse(credit_card.venmo_sdk)
         self.assertNotEqual(re.search("png", credit_card.image_url), None)
 
     def test_create_and_make_default(self):
@@ -93,19 +129,37 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual("05/2014", credit_card.expiration_date)
 
     def test_create_with_security_params(self):
-        customer = Customer.create().customer
-        result = CreditCard.create({
-            "customer_id": customer.id,
-            "number": "4111111111111111",
-            "expiration_month": "05",
-            "expiration_year": "2014",
-            "cvv": "100",
-            "cardholder_name": "John Doe",
-            "device_session_id": "abc123",
-            "fraud_merchant_id": "456"
-        })
+            customer = Customer.create().customer
+            result = CreditCard.create({
+                "customer_id": customer.id,
+                "number": "4111111111111111",
+                "expiration_month": "05",
+                "expiration_year": "2014",
+                "cvv": "100",
+                "cardholder_name": "John Doe",
+                "device_data": "abc123"
+                })
 
-        self.assertTrue(result.is_success)
+            self.assertTrue(result.is_success)
+
+    def test_create_with_deprecated_security_params_sends_warning(self):
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            customer = Customer.create().customer
+            result = CreditCard.create({
+                "customer_id": customer.id,
+                "number": "4111111111111111",
+                "expiration_month": "05",
+                "expiration_year": "2014",
+                "cvv": "100",
+                "cardholder_name": "John Doe",
+                "device_session_id": "abc123",
+                "fraud_merchant_id": "456"
+                })
+
+            self.assertTrue(result.is_success)
+            assert len(w) > 0
+            assert issubclass(w[-1].category, DeprecationWarning)
 
     def test_create_can_specify_the_desired_token(self):
         token = str(random.randint(1, 1000000))
@@ -135,7 +189,8 @@ class TestCreditCard(unittest.TestCase):
                 "country_code_alpha2": "MX",
                 "country_code_alpha3": "MEX",
                 "country_code_numeric": "484",
-                "country_name": "Mexico"
+                "country_name": "Mexico",
+                "phone_number": "312-123-4567"
             }
         })
 
@@ -149,6 +204,7 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual("MEX", address.country_code_alpha3)
         self.assertEqual("484", address.country_code_numeric)
         self.assertEqual("Mexico", address.country_name)
+        self.assertEqual("312-123-4567", address.phone_number)
 
     def test_create_with_billing_address_id(self):
         customer = Customer.create().customer
@@ -179,44 +235,74 @@ class TestCreditCard(unittest.TestCase):
         self.assertTrue(result.is_success)
         self.assertEqual(None, result.credit_card.billing_address)
 
-
     def test_unsuccessful_create_with_card_verification_returns_risk_data(self):
-        with AdvancedFraudIntegrationMerchant():
+        with FraudProtectionEnterpriseIntegrationMerchant():
             customer = Customer.create().customer
             result = CreditCard.create({
                 "customer_id": customer.id,
                 "number": "4000111111111115",
                 "expiration_date": "05/2014",
                 "options": {"verify_card": True},
-                "device_session_id": "abc123"
+                "device_data": "abc123"
             })
 
             self.assertFalse(result.is_success)
             verification = result.credit_card_verification
             self.assertIsInstance(verification.risk_data, RiskData)
             self.assertTrue(hasattr(verification.risk_data, 'id'))
-            self.assertEqual("Approve", verification.risk_data.decision)
-            self.assertTrue(hasattr(verification.risk_data, 'device_data_captured'))
+            self.assertTrue(hasattr(verification.risk_data, 'decision'))
+            self.assertTrue(hasattr(verification.risk_data, 'decision_reasons'))
             self.assertTrue(hasattr(verification.risk_data, 'fraud_service_provider'))
+            self.assertTrue(hasattr(verification.risk_data, 'transaction_risk_score'))
 
     def test_successful_create_with_card_verification_returns_risk_data(self):
-        with AdvancedFraudIntegrationMerchant():
+        with FraudProtectionEnterpriseIntegrationMerchant():
             customer = Customer.create().customer
             result = CreditCard.create({
                 "customer_id": customer.id,
                 "number": "4111111111111111",
                 "expiration_date": "05/2014",
                 "options": {"verify_card": True},
-                "device_session_id": "abc123"
+                "device_data": "abc123"
             })
 
             self.assertTrue(result.is_success)
             verification = result.credit_card.verification
             self.assertIsInstance(verification.risk_data, RiskData)
-            self.assertTrue(hasattr(verification.risk_data, 'id'))
-            self.assertEqual("Approve", verification.risk_data.decision)
-            self.assertTrue(hasattr(verification.risk_data, 'device_data_captured'))
-            self.assertTrue(hasattr(verification.risk_data, 'fraud_service_provider'))
+
+    def test_create_includes_risk_data_when_skip_advanced_fraud_checking_is_false(self):
+        with FraudProtectionEnterpriseIntegrationMerchant():
+            customer = Customer.create().customer
+            result = CreditCard.create({
+                "customer_id": customer.id,
+                "number": "4111111111111111",
+                "expiration_date": "05/2014",
+                "options": {
+                    "verify_card": True,
+                    "skip_advanced_fraud_checking": False
+                    },
+                })
+
+            self.assertTrue(result.is_success)
+            verification = result.credit_card.verification
+            self.assertIsInstance(verification.risk_data, RiskData)
+
+    def test_create_does_not_include_risk_data_when_skip_advanced_fraud_checking_is_true(self):
+        with FraudProtectionEnterpriseIntegrationMerchant():
+            customer = Customer.create().customer
+            result = CreditCard.create({
+                "customer_id": customer.id,
+                "number": "4111111111111111",
+                "expiration_date": "05/2014",
+                "options": {
+                    "verify_card": True,
+                    "skip_advanced_fraud_checking": True
+                    },
+                })
+
+            self.assertTrue(result.is_success)
+            verification = result.credit_card.verification
+            self.assertIsNone(verification.risk_data)
 
     def test_create_with_card_verification(self):
         customer = Customer.create().customer
@@ -421,31 +507,6 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual(1, len(country_name_errors))
         self.assertEqual(ErrorCodes.Address.CountryNameIsNotAccepted, country_name_errors[0].code)
 
-    def test_create_with_venmo_sdk_payment_method_code(self):
-        customer = Customer.create().customer
-        result = CreditCard.create({
-            "customer_id": customer.id,
-            "venmo_sdk_payment_method_code": venmo_sdk.VisaPaymentMethodCode
-        })
-
-        self.assertTrue(result.is_success)
-        self.assertEqual("411111", result.credit_card.bin)
-        self.assertFalse(result.credit_card.venmo_sdk)
-
-    def test_create_with_invalid_venmo_sdk_payment_method_code(self):
-        customer = Customer.create().customer
-        result = CreditCard.create({
-            "customer_id": customer.id,
-            "venmo_sdk_payment_method_code": venmo_sdk.InvalidPaymentMethodCode
-        })
-
-        self.assertFalse(result.is_success)
-        self.assertEqual("Invalid VenmoSDK payment method code", result.message)
-
-        venmo_sdk_payment_method_code_errors = result.errors.for_object("credit_card").on("venmo_sdk_payment_method_code")
-        self.assertEqual(1, len(venmo_sdk_payment_method_code_errors))
-        self.assertEqual(ErrorCodes.CreditCard.InvalidVenmoSDKPaymentMethodCode, venmo_sdk_payment_method_code_errors[0].code)
-
     def test_create_with_payment_method_nonce(self):
         config = Configuration.instantiate()
         authorization_fingerprint = json.loads(TestHelper.generate_decoded_client_token())["authorizationFingerprint"]
@@ -473,35 +534,16 @@ class TestCreditCard(unittest.TestCase):
         self.assertTrue(result.is_success)
         self.assertEqual("411111", result.credit_card.bin)
 
-    def test_create_with_venmo_sdk_session(self):
-        customer = Customer.create().customer
-        result = CreditCard.create({
-            "customer_id": customer.id,
-            "number": "4111111111111111",
-            "expiration_date": "05/2014",
-            "cvv": "100",
-            "cardholder_name": "John Doe",
-            "options": {
-                "venmo_sdk_session": venmo_sdk.Session
-            }
-        })
-        self.assertTrue(result.is_success)
-        self.assertFalse(result.credit_card.venmo_sdk)
+    def test_delete_customer_with_path_traversal(self):
+        try:
+            customer = Customer.create({"first_name":"Waldo"}).customer
+            CreditCard.delete("../../customers/{}".format(customer.id))
+        except NotFoundError:
+            pass
 
-    def test_create_with_invalid_venmo_sdk_session(self):
-        customer = Customer.create().customer
-        result = CreditCard.create({
-            "customer_id": customer.id,
-            "number": "4111111111111111",
-            "expiration_date": "05/2014",
-            "cvv": "100",
-            "cardholder_name": "John Doe",
-            "options": {
-                "venmo_sdk_session": venmo_sdk.InvalidSession
-            }
-        })
-        self.assertTrue(result.is_success)
-        self.assertFalse(result.credit_card.venmo_sdk)
+        found_customer = Customer.find(customer.id)
+        self.assertNotEqual(None, found_customer)
+        self.assertEqual("Waldo", found_customer.first_name)
 
     def test_update_with_valid_options(self):
         customer = Customer.create().customer
@@ -530,6 +572,60 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual("06/2010", credit_card.expiration_date)
         self.assertEqual("Jane Jones", credit_card.cardholder_name)
 
+    def test_update_with_three_d_secure_pass_thru(self):
+        customer = Customer.create().customer
+        credit_card = CreditCard.create({
+            "customer_id": customer.id,
+            "number": "4111111111111111",
+            "expiration_date": "05/2014",
+            "cvv": "100",
+            "cardholder_name": "John Doe"
+        }).credit_card
+
+        result = CreditCard.update(credit_card.token, {
+            "number": "4111111111111111",
+            "expiration_date": "05/2009",
+            "cvv": "123",
+            "three_d_secure_pass_thru": {
+                "three_d_secure_version": "1.1.1",
+                "eci_flag": "05",
+                "cavv": "some-cavv",
+                "xid": "some-xid"
+            },
+            "options": {
+                "verify_card": "true",
+            }
+        })
+
+        self.assertTrue(result.is_success)
+
+    def test_update_with_three_d_secure_pass_thru_without_eci_flag(self):
+        customer = Customer.create().customer
+        credit_card = CreditCard.create({
+            "customer_id": customer.id,
+            "number": "4111111111111111",
+            "expiration_date": "05/2014",
+            "cvv": "100",
+            "cardholder_name": "John Doe"
+        }).credit_card
+
+        result = CreditCard.update(credit_card.token, {
+            "number": "4111111111111111",
+            "expiration_date": "05/2009",
+            "cvv": "123",
+            "three_d_secure_pass_thru": {
+                "three_d_secure_version": "1.1.1",
+                "cavv": "some-cavv",
+                "xid": "some-xid"
+            },
+            "options": {
+                "verify_card": "true",
+            }
+        })
+
+        self.assertFalse(result.is_success)
+        self.assertEqual("EciFlag is required.", result.message)
+
     def test_update_billing_address_creates_new_by_default(self):
         customer = Customer.create().customer
         initial_credit_card = CreditCard.create({
@@ -657,6 +753,52 @@ class TestCreditCard(unittest.TestCase):
         self.assertFalse(result.is_success)
         self.assertEqual(CreditCardVerification.Status.ProcessorDeclined, result.credit_card_verification.status)
 
+    def test_update_includes_risk_data_when_skip_advanced_fraud_checking_is_false(self):
+        with FraudProtectionEnterpriseIntegrationMerchant():
+            customer = Customer.create().customer
+            credit_card = CreditCard.create({
+                "customer_id": customer.id,
+                "number": "4111111111111111",
+                "expiration_date": "05/2014",
+                "cvv": "100",
+                "cardholder_name": "John Doe"
+            }).credit_card
+
+            result = CreditCard.update(credit_card.token, {
+                "expiration_date": "06/2020",
+                "options": {
+                    "verify_card": True,
+                    "skip_advanced_fraud_checking": False
+                    }
+            })
+
+            self.assertTrue(result.is_success)
+            verification = result.credit_card.verification
+            self.assertIsInstance(verification.risk_data, RiskData)
+
+    def test_update_does_not_include_risk_data_when_skip_advanced_fraud_checking_is_true(self):
+        with FraudProtectionEnterpriseIntegrationMerchant():
+            customer = Customer.create().customer
+            credit_card = CreditCard.create({
+                "customer_id": customer.id,
+                "number": "4111111111111111",
+                "expiration_date": "05/2014",
+                "cvv": "100",
+                "cardholder_name": "John Doe"
+            }).credit_card
+
+            result = CreditCard.update(credit_card.token, {
+                "expiration_date": "06/2020",
+                "options": {
+                    "verify_card": True,
+                    "skip_advanced_fraud_checking": True
+                    }
+            })
+
+            self.assertTrue(result.is_success)
+            verification = result.credit_card.verification
+            self.assertIsNone(verification.risk_data)
+
     def test_update_billing_address(self):
         customer = Customer.create().customer
         credit_card = CreditCard.create({
@@ -740,21 +882,21 @@ class TestCreditCard(unittest.TestCase):
         result = CreditCard.delete(credit_card.token)
         self.assertTrue(result.is_success)
 
-    @raises(NotFoundError)
     def test_delete_raises_error_when_deleting_twice(self):
-        customer = Customer.create().customer
-        credit_card = CreditCard.create({
-            "customer_id": customer.id,
-            "number": "4111111111111111",
-            "expiration_date": "05/2014"
-        }).credit_card
+        with self.assertRaises(NotFoundError):
+            customer = Customer.create().customer
+            credit_card = CreditCard.create({
+                "customer_id": customer.id,
+                "number": "4111111111111111",
+                "expiration_date": "05/2014"
+            }).credit_card
 
-        CreditCard.delete(credit_card.token)
-        CreditCard.delete(credit_card.token)
+            CreditCard.delete(credit_card.token)
+            CreditCard.delete(credit_card.token)
 
-    @raises(NotFoundError)
     def test_delete_with_invalid_token(self):
-        CreditCard.delete("notreal")
+        with self.assertRaises(NotFoundError):
+            CreditCard.delete("notreal")
 
     def test_find_with_valid_token(self):
         customer = Customer.create().customer
@@ -797,9 +939,9 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual(Decimal("1.00"), subscription.price)
         self.assertEqual(credit_card.token, subscription.payment_method_token)
 
-    @raises_with_regexp(NotFoundError, "payment method with token 'bad_token' not found")
     def test_find_with_invalid_token(self):
-        CreditCard.find("bad_token")
+        with self.assertRaisesRegex(NotFoundError, "payment method with token 'bad_token' not found"):
+            CreditCard.find("bad_token")
 
     def test_from_nonce_with_unlocked_nonce(self):
         config = Configuration.instantiate()
@@ -830,310 +972,69 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual(1, len(customer.credit_cards))
         self.assertEqual(customer.credit_cards[0].token, card.token)
 
-    @raises_with_regexp(NotFoundError, "payment method with nonce .* or not found")
     def test_from_nonce_with_unlocked_nonce_pointing_to_shared_card(self):
-        config = Configuration.instantiate()
+        with self.assertRaisesRegex(NotFoundError, "payment method with nonce .* or not found"):
+            config = Configuration.instantiate()
 
-        client_token = TestHelper.generate_decoded_client_token()
-        authorization_fingerprint = json.loads(client_token)["authorizationFingerprint"]
-        http = ClientApiHttp(config, {
-            "authorization_fingerprint": authorization_fingerprint,
-            "shared_customer_identifier": "fake_identifier",
-            "shared_customer_identifier_type": "testing"
-        })
+            client_token = TestHelper.generate_decoded_client_token()
+            authorization_fingerprint = json.loads(client_token)["authorizationFingerprint"]
+            http = ClientApiHttp(config, {
+                "authorization_fingerprint": authorization_fingerprint,
+                "shared_customer_identifier": "fake_identifier",
+                "shared_customer_identifier_type": "testing"
+            })
 
-        status_code, response = http.add_card({
-            "credit_card": {
-                "number": "4111111111111111",
-                "expiration_month": "11",
-                "expiration_year": "2099",
-            },
-            "share": True
-        })
-        self.assertEqual(201, status_code)
-        nonce = json.loads(response)["creditCards"][0]["nonce"]
+            status_code, response = http.add_card({
+                "credit_card": {
+                    "number": "4111111111111111",
+                    "expiration_month": "11",
+                    "expiration_year": "2099",
+                },
+                "share": True
+            })
+            self.assertEqual(201, status_code)
+            nonce = json.loads(response)["creditCards"][0]["nonce"]
+
+            CreditCard.from_nonce(nonce)
 
-        CreditCard.from_nonce(nonce)
 
-    @raises_with_regexp(NotFoundError, ".* consumed .*")
     def test_from_nonce_with_consumed_nonce(self):
-        config = Configuration.instantiate()
-        customer = Customer.create().customer
+        with self.assertRaisesRegex(NotFoundError, ".* consumed .*"):
+            config = Configuration.instantiate()
+            customer = Customer.create().customer
 
-        client_token = TestHelper.generate_decoded_client_token({
-            "customer_id": customer.id,
-        })
-        authorization_fingerprint = json.loads(client_token)["authorizationFingerprint"]
-        http = ClientApiHttp(config, {
-            "authorization_fingerprint": authorization_fingerprint,
-            "shared_customer_identifier": "fake_identifier",
-            "shared_customer_identifier_type": "testing"
-        })
-
-        status_code, response = http.add_card({
-            "credit_card": {
-                "number": "4111111111111111",
-                "expiration_month": "11",
-                "expiration_year": "2099",
-            }
-        })
-        self.assertEqual(201, status_code)
-        nonce = json.loads(response)["creditCards"][0]["nonce"]
-
-        CreditCard.from_nonce(nonce)
-        CreditCard.from_nonce(nonce)
-
-    def test_create_from_transparent_redirect(self):
-        customer = Customer.create().customer
-        tr_data = {
-            "credit_card": {
-                "customer_id": customer.id
-            }
-        }
-        post_params = {
-            "tr_data": CreditCard.tr_data_for_create(tr_data, "http://example.com/path?foo=bar"),
-            "credit_card[cardholder_name]": "Card Holder",
-            "credit_card[number]": "4111111111111111",
-            "credit_card[expiration_date]": "05/2012",
-            "credit_card[billing_address][country_code_alpha2]": "MX",
-            "credit_card[billing_address][country_code_alpha3]": "MEX",
-            "credit_card[billing_address][country_code_numeric]": "484",
-            "credit_card[billing_address][country_name]": "Mexico",
-        }
-
-        query_string = TestHelper.simulate_tr_form_post(post_params, CreditCard.transparent_redirect_create_url())
-        result = CreditCard.confirm_transparent_redirect(query_string)
-        self.assertTrue(result.is_success)
-        credit_card = result.credit_card
-        self.assertEqual("411111", credit_card.bin)
-        self.assertEqual("1111", credit_card.last_4)
-        self.assertEqual("05", credit_card.expiration_month)
-        self.assertEqual("2012", credit_card.expiration_year)
-        self.assertEqual(customer.id, credit_card.customer_id)
-        self.assertEqual("MX", credit_card.billing_address.country_code_alpha2)
-        self.assertEqual("MEX", credit_card.billing_address.country_code_alpha3)
-        self.assertEqual("484", credit_card.billing_address.country_code_numeric)
-        self.assertEqual("Mexico", credit_card.billing_address.country_name)
-
-
-    def test_create_from_transparent_redirect_and_make_default(self):
-        customer = Customer.create().customer
-        card1 = CreditCard.create({
-            "customer_id": customer.id,
-            "number": "4111111111111111",
-            "expiration_date": "05/2014",
-            "cvv": "100",
-            "cardholder_name": "John Doe"
-        }).credit_card
-        self.assertTrue(card1.default)
-
-        tr_data = {
-            "credit_card": {
+            client_token = TestHelper.generate_decoded_client_token({
                 "customer_id": customer.id,
-                "options": {
-                    "make_default": True
+            })
+            authorization_fingerprint = json.loads(client_token)["authorizationFingerprint"]
+            http = ClientApiHttp(config, {
+                "authorization_fingerprint": authorization_fingerprint,
+                "shared_customer_identifier": "fake_identifier",
+                "shared_customer_identifier_type": "testing"
+            })
+
+            status_code, response = http.add_card({
+                "credit_card": {
+                    "number": "4111111111111111",
+                    "expiration_month": "11",
+                    "expiration_year": "2099",
                 }
-            }
-        }
-        post_params = {
-            "tr_data": CreditCard.tr_data_for_create(tr_data, "http://example.com/path?foo=bar"),
-            "credit_card[cardholder_name]": "Card Holder",
-            "credit_card[number]": "4111111111111111",
-            "credit_card[expiration_date]": "05/2012",
-        }
+            })
+            self.assertEqual(201, status_code)
+            nonce = json.loads(response)["creditCards"][0]["nonce"]
 
-        query_string = TestHelper.simulate_tr_form_post(post_params, CreditCard.transparent_redirect_create_url())
-        card2 = CreditCard.confirm_transparent_redirect(query_string).credit_card
-
-        self.assertFalse(CreditCard.find(card1.token).default)
-        self.assertTrue(card2.default)
-
-    def test_create_from_transparent_redirect_with_error_result(self):
-        customer = Customer.create().customer
-        tr_data = {
-            "credit_card": {
-                "customer_id": customer.id
-            }
-        }
-
-        post_params = {
-            "tr_data": CreditCard.tr_data_for_create(tr_data, "http://example.com/path"),
-            "credit_card[cardholder_name]": "Card Holder",
-            "credit_card[number]": "eleventy",
-            "credit_card[expiration_date]": "y2k"
-        }
-
-        query_string = TestHelper.simulate_tr_form_post(post_params, CreditCard.transparent_redirect_create_url())
-        result = CreditCard.confirm_transparent_redirect(query_string)
-        self.assertFalse(result.is_success)
-
-        credit_card_number_errors = result.errors.for_object("credit_card").on("number")
-        self.assertEqual(1, len(credit_card_number_errors))
-        self.assertEqual(ErrorCodes.CreditCard.NumberHasInvalidLength, credit_card_number_errors[0].code)
-
-        expiration_date_errors = result.errors.for_object("credit_card").on("expiration_date")
-        self.assertEqual(1, len(expiration_date_errors))
-        self.assertEqual(ErrorCodes.CreditCard.ExpirationDateIsInvalid, expiration_date_errors[0].code)
-
-    def test_update_from_transparent_redirect_with_successful_result(self):
-        old_token = str(random.randint(1, 1000000))
-        new_token = str(random.randint(1, 1000000))
-        credit_card = Customer.create({
-            "credit_card": {
-                "cardholder_name": "Old Cardholder Name",
-                "number": "4111111111111111",
-                "expiration_date": "05/2012",
-                "token": old_token
-            }
-        }).customer.credit_cards[0]
-
-        tr_data = {
-            "payment_method_token": old_token,
-            "credit_card": {
-                "token": new_token
-            }
-        }
-
-        post_params = {
-            "tr_data": CreditCard.tr_data_for_update(tr_data, "http://example.com/path"),
-            "credit_card[cardholder_name]": "New Cardholder Name",
-            "credit_card[expiration_date]": "05/2014"
-        }
-
-        query_string = TestHelper.simulate_tr_form_post(post_params, CreditCard.transparent_redirect_update_url())
-        result = CreditCard.confirm_transparent_redirect(query_string)
-        self.assertTrue(result.is_success)
-        credit_card = result.credit_card
-        self.assertEqual(new_token, credit_card.token)
-        self.assertEqual("411111", credit_card.bin)
-        self.assertEqual("1111", credit_card.last_4)
-        self.assertEqual("05", credit_card.expiration_month)
-        self.assertEqual("2014", credit_card.expiration_year)
-
-    def test_update_from_transparent_redirect_and_make_default(self):
-        customer = Customer.create({
-            "credit_card": {
-                "number": "4111111111111111",
-                "expiration_date": "05/2012"
-            }
-        }).customer
-        card1 = customer.credit_cards[0]
-
-        card2 = CreditCard.create({
-            "customer_id": customer.id,
-            "number": "4111111111111111",
-            "expiration_date": "05/2014",
-        }).credit_card
-
-        self.assertTrue(card1.default)
-        self.assertFalse(card2.default)
-
-        tr_data = {
-            "payment_method_token": card2.token,
-            "credit_card": {
-                "options": {
-                    "make_default": True
-                }
-            }
-        }
-
-        post_params = {
-            "tr_data": CreditCard.tr_data_for_update(tr_data, "http://example.com/path"),
-            "credit_card[cardholder_name]": "New Cardholder Name",
-            "credit_card[expiration_date]": "05/2014"
-        }
-
-        query_string = TestHelper.simulate_tr_form_post(post_params, CreditCard.transparent_redirect_update_url())
-        CreditCard.confirm_transparent_redirect(query_string)
-
-        self.assertFalse(CreditCard.find(card1.token).default)
-        self.assertTrue(CreditCard.find(card2.token).default)
-
-    def test_update_from_transparent_redirect_and_update_existing_billing_address(self):
-        customer = Customer.create({
-            "credit_card": {
-                "number": "4111111111111111",
-                "expiration_date": "05/2012",
-                "billing_address": {
-                    "street_address": "123 Old St",
-                    "locality": "Chicago",
-                    "region": "Illinois",
-                    "postal_code": "60621"
-                }
-            }
-        }).customer
-        card = customer.credit_cards[0]
-
-        tr_data = {
-            "payment_method_token": card.token,
-            "credit_card": {
-                "billing_address": {
-                    "street_address": "123 New St",
-                    "locality": "Columbus",
-                    "region": "Ohio",
-                    "postal_code": "43215",
-                    "options": {
-                        "update_existing": True
-                    }
-                }
-            }
-        }
-
-        post_params = {
-            "tr_data": CreditCard.tr_data_for_update(tr_data, "http://example.com/path")
-        }
-
-        query_string = TestHelper.simulate_tr_form_post(post_params, CreditCard.transparent_redirect_update_url())
-
-        CreditCard.confirm_transparent_redirect(query_string)
-
-        self.assertEqual(1, len(Customer.find(customer.id).addresses))
-        updated_card = CreditCard.find(card.token)
-        self.assertEqual("123 New St", updated_card.billing_address.street_address)
-        self.assertEqual("Columbus", updated_card.billing_address.locality)
-        self.assertEqual("Ohio", updated_card.billing_address.region)
-        self.assertEqual("43215", updated_card.billing_address.postal_code)
-
-    def test_update_from_transparent_redirect_with_error_result(self):
-        old_token = str(random.randint(1, 1000000))
-        Customer.create({
-            "credit_card": {
-                "cardholder_name": "Old Cardholder Name",
-                "number": "4111111111111111",
-                "expiration_date": "05/2012",
-                "token": old_token
-            }
-        })
-
-        tr_data = {
-            "payment_method_token": old_token,
-            "credit_card": {
-                "token": "bad token"
-            }
-        }
-
-        post_params = {
-            "tr_data": CreditCard.tr_data_for_update(tr_data, "http://example.com/path"),
-            "credit_card[cardholder_name]": "New Cardholder Name",
-            "credit_card[expiration_date]": "05/2014"
-        }
-
-        query_string = TestHelper.simulate_tr_form_post(post_params, CreditCard.transparent_redirect_update_url())
-        result = CreditCard.confirm_transparent_redirect(query_string)
-        self.assertFalse(result.is_success)
-
-        credit_card_token_errors = result.errors.for_object("credit_card").on("token")
-        self.assertEqual(1, len(credit_card_token_errors))
-        self.assertEqual(ErrorCodes.CreditCard.TokenInvalid, credit_card_token_errors[0].code)
+            CreditCard.from_nonce(nonce)
+            CreditCard.from_nonce(nonce)
 
     def test_expired_can_iterate_over_all_items(self):
+        year = datetime.now().year - 3
         customer_id = Customer.all().first.id
 
         for _ in range(110 - CreditCard.expired().maximum_size):
             CreditCard.create({
                 "customer_id": customer_id,
                 "number": "4111111111111111",
-                "expiration_date": "01/2015"
+                "expiration_date": "01/" + str(year)
             })
 
         collection = CreditCard.expired()
@@ -1307,7 +1208,7 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual(CreditCard.Healthcare.Unknown, credit_card.healthcare)
         self.assertEqual(CreditCard.IssuingBank.Unknown, credit_card.issuing_bank)
         self.assertEqual(CreditCard.CountryOfIssuance.Unknown, credit_card.country_of_issuance)
-        self.assertEquals(CreditCard.ProductId.Unknown, credit_card.product_id)
+        self.assertEqual(CreditCard.ProductId.Unknown, credit_card.product_id)
 
     def test_card_with_account_type_debit(self):
         customer = Customer.create().customer
@@ -1390,3 +1291,22 @@ class TestCreditCard(unittest.TestCase):
         self.assertEqual(updated_result.is_success, True)
         self.assertEqual("debit", updated_result.credit_card.verifications[0]["credit_card"]["account_type"])
         self.assertEqual("credit", updated_result.credit_card.verifications[1]["credit_card"]["account_type"])
+
+    def test_network_tokenized_credit_card(self):
+        credit_card = CreditCard.find("network_tokenized_credit_card")
+
+        self.assertEqual(credit_card.is_network_tokenized, True)
+
+    def test_non_network_tokenized_credit_card(self):
+        customer = Customer.create().customer
+        card = CreditCard.create({
+            "customer_id": customer.id,
+            "number": "4111111111111111",
+            "expiration_date": "05/2014",
+            "cvv": "100",
+            "cardholder_name": "John Doe"
+        }).credit_card
+
+        credit_card = CreditCard.find(card.token)
+
+        self.assertEqual(credit_card.is_network_tokenized, False)
