@@ -1,35 +1,12 @@
 from tests.test_helper import *
 
 class TestMerchantAccount(unittest.TestCase):
-    DEPRECATED_APPLICATION_PARAMS = {
-        "applicant_details": {
-            "company_name": "Garbage Garage",
-            "first_name": "Joe",
-            "last_name": "Bloggs",
-            "email": "joe@bloggs.com",
-            "phone": "555-555-5555",
-            "address": {
-                "street_address": "123 Credibility St.",
-                "postal_code": "60606",
-                "locality": "Chicago",
-                "region": "IL",
-            },
-            "date_of_birth": "10/9/1980",
-            "ssn": "123-00-1234",
-            "tax_id": "123456789",
-            "routing_number": "122100024",
-            "account_number": "43759348798"
-        },
-        "tos_accepted": True,
-        "master_merchant_account_id": "sandbox_master_merchant_account"
-    }
-
     VALID_APPLICATION_PARAMS = {
         "individual": {
             "first_name": "Joe",
             "last_name": "Bloggs",
             "email": "joe@bloggs.com",
-            "phone": "555-555-5555",
+            "phone": "555-123-1234",
             "address": {
                 "street_address": "123 Credibility St.",
                 "postal_code": "60606",
@@ -42,7 +19,7 @@ class TestMerchantAccount(unittest.TestCase):
         "business": {
             "dba_name": "Garbage Garage",
             "legal_name": "Junk Jymnasium",
-            "tax_id": "123456789",
+            "tax_id": "423456789",
             "address": {
                 "street_address": "123 Reputation St.",
                 "postal_code": "40222",
@@ -60,13 +37,6 @@ class TestMerchantAccount(unittest.TestCase):
         "master_merchant_account_id": "sandbox_master_merchant_account"
     }
 
-    def test_create_accepts_deprecated_parameters(self):
-        result = MerchantAccount.create(self.DEPRECATED_APPLICATION_PARAMS)
-
-        self.assertTrue(result.is_success)
-        self.assertEqual(MerchantAccount.Status.Pending, result.merchant_account.status)
-        self.assertEqual("sandbox_master_merchant_account", result.merchant_account.master_merchant_account.id)
-
     def test_create_application_with_valid_params_and_no_id(self):
         result = MerchantAccount.create(self.VALID_APPLICATION_PARAMS)
 
@@ -95,15 +65,12 @@ class TestMerchantAccount(unittest.TestCase):
 
     def test_create_requires_all_fields(self):
         result = MerchantAccount.create(
-            {"master_merchant_account_id": "sandbox_master_merchant_account",
-             "applicant_details": {},
-            "tos_accepted": True}
+            {"master_merchant_account_id": "sandbox_master_merchant_account"}
         )
         self.assertFalse(result.is_success)
 
-        first_name_errors = result.errors.for_object("merchant_account").for_object("applicant_details").on("first_name")
-        self.assertEqual(1, len(first_name_errors))
-        self.assertEqual(ErrorCodes.MerchantAccount.ApplicantDetails.FirstNameIsRequired, first_name_errors[0].code)
+        tos_errors = result.errors.for_object("merchant_account").on("tos_accepted")
+        self.assertEqual(ErrorCodes.MerchantAccount.TosAcceptedIsRequired , tos_errors[0].code)
 
     def test_create_funding_destination_accepts_a_bank(self):
         params = self.VALID_APPLICATION_PARAMS.copy()
@@ -144,7 +111,7 @@ class TestMerchantAccount(unittest.TestCase):
             "business": {
                 "dba_name": "James's Bloggs",
                 "legal_name": "James's Junkyard",
-                "tax_id": "987654321",
+                "tax_id": "987651324",
                 "address": {
                     "street_address": "456 Fake St",
                     "postal_code": "48104",
@@ -178,7 +145,7 @@ class TestMerchantAccount(unittest.TestCase):
         self.assertEqual(result.merchant_account.individual_details.address_details.postal_code, "60622")
         self.assertEqual(result.merchant_account.business_details.dba_name, "James's Bloggs")
         self.assertEqual(result.merchant_account.business_details.legal_name, "James's Junkyard")
-        self.assertEqual(result.merchant_account.business_details.tax_id, "987654321")
+        self.assertEqual(result.merchant_account.business_details.tax_id, "987651324")
         self.assertEqual(result.merchant_account.business_details.address_details.street_address, "456 Fake St")
         self.assertEqual(result.merchant_account.business_details.address_details.postal_code, "48104")
         self.assertEqual(result.merchant_account.business_details.address_details.locality, "Ann Arbor")
@@ -428,9 +395,9 @@ class TestMerchantAccount(unittest.TestCase):
         self.assertEqual(merchant_account.status, MerchantAccount.Status.Active)
         self.assertTrue(merchant_account.default)
 
-    @raises(NotFoundError)
     def test_find_404(self):
-        MerchantAccount.find("not_a_real_id")
+        with self.assertRaises(NotFoundError):
+            MerchantAccount.find("not_a_real_id")
 
     def test_merchant_account_create_for_currency(self):
         self.gateway = BraintreeGateway(
