@@ -19,27 +19,62 @@ class TestPaymentMethod(unittest.TestCase):
 
         three_d_secure_info = result.payment_method.verification.three_d_secure_info
 
-        self.assertEqual("Y", three_d_secure_info.enrolled)
         self.assertEqual("authenticate_successful", three_d_secure_info.status)
         self.assertEqual(True, three_d_secure_info.liability_shifted)
         self.assertEqual(True, three_d_secure_info.liability_shift_possible)
-        self.assertEqual("cavv_value", three_d_secure_info.cavv)
-        self.assertEqual("xid_value", three_d_secure_info.xid)
-        self.assertEqual(None, three_d_secure_info.ds_transaction_id)
-        self.assertEqual("05", three_d_secure_info.eci_flag)
-        self.assertEqual("1.0.2", three_d_secure_info.three_d_secure_version)
+        self.assertIsInstance(three_d_secure_info.enrolled, str)
+        self.assertIsInstance(three_d_secure_info.cavv, str)
+        self.assertIsInstance(three_d_secure_info.xid, str)
+        self.assertIsInstance(three_d_secure_info.eci_flag, str)
+        self.assertIsInstance(three_d_secure_info.three_d_secure_version, str)
 
-    def test_create_with_paypal_future_payments_nonce(self):
+    def test_create_with_three_d_secure_pass_thru(self):
         customer_id = Customer.create().customer.id
         result = PaymentMethod.create({
             "customer_id": customer_id,
-            "payment_method_nonce": Nonces.PayPalFuturePayment
+            "payment_method_nonce": Nonces.Transactable,
+            "three_d_secure_pass_thru": {
+                "three_d_secure_version": "1.1.0",
+                "eci_flag": "05",
+                "cavv": "some-cavv",
+                "xid": "some-xid"
+            },
+            "options": {
+                "verify_card": "true",
+            }
+        })
+
+        self.assertTrue(result.is_success)
+
+    def test_create_with_three_d_secure_pass_thru_without_eci_flag(self):
+        customer_id = Customer.create().customer.id
+        result = PaymentMethod.create({
+            "customer_id": customer_id,
+            "payment_method_nonce": Nonces.Transactable,
+            "three_d_secure_pass_thru": {
+                "three_d_secure_version": "1.1.0",
+                "cavv": "some-cavv",
+                "xid": "some-xid"
+            },
+            "options": {
+                "verify_card": "true",
+            }
+        })
+
+        self.assertFalse(result.is_success)
+        self.assertEqual("EciFlag is required.", result.message)
+
+    def test_create_with_paypal_billing_agreements_nonce(self):
+        customer_id = Customer.create().customer.id
+        result = PaymentMethod.create({
+            "customer_id": customer_id,
+            "payment_method_nonce": Nonces.PayPalBillingAgreement
         })
 
         self.assertTrue(result.is_success)
         created_account = result.payment_method
         self.assertEqual(PayPalAccount, created_account.__class__)
-        self.assertEqual("jane.doe@example.com", created_account.email)
+        self.assertEqual("jane.doe@paypal.com", created_account.email)
         self.assertNotEqual(created_account.image_url, None)
 
         found_account = PaymentMethod.find(result.payment_method.token)
@@ -118,25 +153,6 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertEqual(created_account.billing_agreement_id, found_account.billing_agreement_id)
         self.assertEqual(created_account.payer_id, found_account.payer_id)
 
-    def test_create_with_paypal_refresh_token_without_upgrade(self):
-        customer_id = Customer.create().customer.id
-        result = PaymentMethod.create({
-            "customer_id": customer_id,
-            "paypal_refresh_token": "PAYPAL_REFRESH_TOKEN",
-            "paypal_vault_without_upgrade": True,
-        })
-
-        self.assertTrue(result.is_success)
-        created_account = result.payment_method
-        self.assertEqual(PayPalAccount, created_account.__class__)
-        self.assertEqual(created_account.billing_agreement_id, None)
-
-        found_account = PaymentMethod.find(result.payment_method.token)
-        self.assertNotEqual(None, found_account)
-        self.assertEqual(created_account.token, found_account.token)
-        self.assertEqual(created_account.customer_id, found_account.customer_id)
-        self.assertEqual(created_account.billing_agreement_id, found_account.billing_agreement_id)
-
     def test_create_returns_validation_failures(self):
         http = ClientApiHttp.create()
         status_code, nonce = http.get_paypal_nonce({
@@ -239,6 +255,18 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertIsInstance(apple_pay_card, ApplePayCard)
         self.assertNotEqual(apple_pay_card.bin, None)
         self.assertNotEqual(apple_pay_card.token, None)
+        self.assertNotEqual(apple_pay_card.prepaid, None)
+        self.assertNotEqual(apple_pay_card.healthcare, None)
+        self.assertNotEqual(apple_pay_card.debit, None)
+        self.assertNotEqual(apple_pay_card.durbin_regulated, None)
+        self.assertNotEqual(apple_pay_card.commercial, None)
+        self.assertNotEqual(apple_pay_card.payroll, None)
+        self.assertNotEqual(apple_pay_card.issuing_bank, None)
+        self.assertNotEqual(apple_pay_card.country_of_issuance, None)
+        self.assertNotEqual(apple_pay_card.product_id, None)
+        self.assertNotEqual(apple_pay_card.last_4, None)
+        self.assertNotEqual(apple_pay_card.card_type, None)
+
         self.assertEqual(apple_pay_card.customer_id, customer_id)
         self.assertEqual(ApplePayCard.CardType.MasterCard, apple_pay_card.card_type)
         self.assertEqual("MasterCard 0017", apple_pay_card.payment_instrument_name)
@@ -248,6 +276,41 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertTrue(int(apple_pay_card.expiration_month) > 0)
         self.assertTrue(int(apple_pay_card.expiration_year) > 0)
 
+    def test_create_with_fake_apple_pay_mpan_nonce(self):
+        customer_id = Customer.create().customer.id
+        result = PaymentMethod.create({
+            "customer_id": customer_id,
+            "payment_method_nonce": Nonces.ApplePayMpan
+        })
+
+        self.assertTrue(result.is_success)
+        apple_pay_card = result.payment_method
+        self.assertIsInstance(apple_pay_card, ApplePayCard)
+        self.assertNotEqual(apple_pay_card.bin, None)
+        self.assertNotEqual(apple_pay_card.token, None)
+        self.assertNotEqual(apple_pay_card.prepaid, None)
+        self.assertNotEqual(apple_pay_card.healthcare, None)
+        self.assertNotEqual(apple_pay_card.debit, None)
+        self.assertNotEqual(apple_pay_card.durbin_regulated, None)
+        self.assertNotEqual(apple_pay_card.commercial, None)
+        self.assertNotEqual(apple_pay_card.payroll, None)
+        self.assertNotEqual(apple_pay_card.issuing_bank, None)
+        self.assertNotEqual(apple_pay_card.country_of_issuance, None)
+        self.assertNotEqual(apple_pay_card.product_id, None)
+        self.assertNotEqual(apple_pay_card.last_4, None)
+        self.assertNotEqual(apple_pay_card.card_type, None)
+        self.assertNotEqual(apple_pay_card.merchant_token_identifier, None)
+        self.assertNotEqual(apple_pay_card.source_card_last4, None)
+
+        self.assertEqual(apple_pay_card.customer_id, customer_id)
+        self.assertEqual(ApplePayCard.CardType.Visa, apple_pay_card.card_type)
+        self.assertEqual("Visa 8886", apple_pay_card.payment_instrument_name)
+        self.assertEqual("Visa 8886", apple_pay_card.source_description)
+        self.assertTrue(apple_pay_card.default)
+        self.assertIn("apple_pay", apple_pay_card.image_url)
+        self.assertTrue(int(apple_pay_card.expiration_month) > 0)
+        self.assertTrue(int(apple_pay_card.expiration_year) > 0)
+
     def test_create_with_fake_android_pay_proxy_card_nonce(self):
         customer_id = Customer.create().customer.id
         result = PaymentMethod.create({
@@ -275,6 +338,7 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertIsInstance(android_pay_card.updated_at, datetime)
         self.assertEqual("601111", android_pay_card.bin)
         self.assertEqual("google_transaction_id", android_pay_card.google_transaction_id)
+        self.assertFalse(android_pay_card.is_network_tokenized)
 
     def test_create_with_fake_android_pay_network_token_nonce(self):
         customer_id = Customer.create().customer.id
@@ -303,6 +367,20 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertIsInstance(android_pay_card.updated_at, datetime)
         self.assertEqual("555555", android_pay_card.bin)
         self.assertEqual("google_transaction_id", android_pay_card.google_transaction_id)
+        self.assertTrue(android_pay_card.is_network_tokenized)
+        self.assertNotEqual(android_pay_card.bin, None)
+        self.assertNotEqual(android_pay_card.token, None)
+        self.assertNotEqual(android_pay_card.prepaid, None)
+        self.assertNotEqual(android_pay_card.healthcare, None)
+        self.assertNotEqual(android_pay_card.debit, None)
+        self.assertNotEqual(android_pay_card.durbin_regulated, None)
+        self.assertNotEqual(android_pay_card.commercial, None)
+        self.assertNotEqual(android_pay_card.payroll, None)
+        self.assertNotEqual(android_pay_card.issuing_bank, None)
+        self.assertNotEqual(android_pay_card.country_of_issuance, None)
+        self.assertNotEqual(android_pay_card.product_id, None)
+        self.assertNotEqual(android_pay_card.last_4, None)
+        self.assertNotEqual(android_pay_card.card_type, None)
 
     def test_create_with_fake_amex_express_checkout_card_nonce(self):
         customer_id = Customer.create().customer.id
@@ -317,13 +395,13 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertNotEqual(amex_express_checkout_card.token, None)
         self.assertTrue(amex_express_checkout_card.default)
         self.assertEqual("American Express", amex_express_checkout_card.card_type)
-        self.assertRegexpMatches(amex_express_checkout_card.bin, r"\A\d{6}\Z")
-        self.assertRegexpMatches(amex_express_checkout_card.expiration_month, r"\A\d{2}\Z")
-        self.assertRegexpMatches(amex_express_checkout_card.expiration_year, r"\A\d{4}\Z")
-        self.assertRegexpMatches(amex_express_checkout_card.card_member_number, r"\A\d{4}\Z")
-        self.assertRegexpMatches(amex_express_checkout_card.card_member_expiry_date, r"\A\d{2}/\d{2}\Z")
-        self.assertRegexpMatches(amex_express_checkout_card.source_description, r"\AAmEx \d{4}\Z")
-        self.assertRegexpMatches(amex_express_checkout_card.image_url, r"\.png")
+        self.assertRegex(amex_express_checkout_card.bin, r"\A\d{6}\Z")
+        self.assertRegex(amex_express_checkout_card.expiration_month, r"\A\d{2}\Z")
+        self.assertRegex(amex_express_checkout_card.expiration_year, r"\A\d{4}\Z")
+        self.assertRegex(amex_express_checkout_card.card_member_number, r"\A\d{4}\Z")
+        self.assertRegex(amex_express_checkout_card.card_member_expiry_date, r"\A\d{2}/\d{2}\Z")
+        self.assertRegex(amex_express_checkout_card.source_description, r"\AAmEx \d{4}\Z")
+        self.assertRegex(amex_express_checkout_card.image_url, r"\.png")
         self.assertEqual(customer_id, amex_express_checkout_card.customer_id)
 
     def test_create_with_venmo_account_nonce(self):
@@ -340,9 +418,9 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertTrue(venmo_account.default)
         self.assertIsNotNone(venmo_account.token)
         self.assertEqual("venmojoe", venmo_account.username)
-        self.assertEqual("Venmo-Joe-1", venmo_account.venmo_user_id)
+        self.assertEqual("1234567891234567891", venmo_account.venmo_user_id)
         self.assertEqual("Venmo Account: venmojoe", venmo_account.source_description)
-        self.assertRegexpMatches(venmo_account.image_url, r"\.png")
+        self.assertRegex(venmo_account.image_url, r"\.png")
         self.assertEqual(customer_id, venmo_account.customer_id)
 
     def test_create_with_abstract_payment_method_nonce(self):
@@ -420,6 +498,68 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertTrue(result.credit_card_verification.processor_response_text == "Do Not Honor")
         self.assertTrue(result.credit_card_verification.merchant_account_id == TestHelper.non_default_merchant_account_id)
 
+    def test_create_includes_risk_data_when_skip_advanced_fraud_checking_is_false(self):
+        with FraudProtectionEnterpriseIntegrationMerchant():
+            config = Configuration.instantiate()
+            customer_id = Customer.create().customer.id
+            client_token = json.loads(TestHelper.generate_decoded_client_token())
+            authorization_fingerprint = client_token["authorizationFingerprint"]
+            http = ClientApiHttp(config, {
+                "authorization_fingerprint": authorization_fingerprint,
+                "shared_customer_identifier": "fake_identifier",
+                "shared_customer_identifier_type": "testing",
+                })
+            status_code, nonce = http.get_credit_card_nonce({
+                "number": "4111111111111111",
+                "expirationMonth": "11",
+                "expirationYear": "2099",
+                })
+            self.assertTrue(status_code == 201)
+
+            result = PaymentMethod.create({
+                "customer_id": customer_id,
+                "payment_method_nonce": nonce,
+                "options": {
+                    "verify_card": True,
+                    "skip_advanced_fraud_checking": False
+                    },
+                })
+
+            self.assertTrue(result.is_success)
+            verification = result.payment_method.verification
+            self.assertIsInstance(verification.risk_data, RiskData)
+
+    def test_create_does_not_include_risk_data_when_skip_advanced_fraud_checking_is_true(self):
+        with FraudProtectionEnterpriseIntegrationMerchant():
+            config = Configuration.instantiate()
+            customer_id = Customer.create().customer.id
+            client_token = json.loads(TestHelper.generate_decoded_client_token())
+            authorization_fingerprint = client_token["authorizationFingerprint"]
+            http = ClientApiHttp(config, {
+                "authorization_fingerprint": authorization_fingerprint,
+                "shared_customer_identifier": "fake_identifier",
+                "shared_customer_identifier_type": "testing",
+                })
+            status_code, nonce = http.get_credit_card_nonce({
+                "number": "4111111111111111",
+                "expirationMonth": "11",
+                "expirationYear": "2099",
+                })
+            self.assertTrue(status_code == 201)
+
+            result = PaymentMethod.create({
+                "customer_id": customer_id,
+                "payment_method_nonce": nonce,
+                "options": {
+                    "verify_card": True,
+                    "skip_advanced_fraud_checking": True
+                    },
+                })
+
+            self.assertTrue(result.is_success)
+            verification = result.payment_method.verification
+            self.assertIsNone(verification.risk_data)
+
     def test_create_respects_fail_one_duplicate_payment_method_when_included_outside_of_the_nonce(self):
         customer_id = Customer.create().customer.id
         credit_card_result = CreditCard.create({
@@ -504,7 +644,8 @@ class TestPaymentMethod(unittest.TestCase):
             "payment_method_nonce": nonce,
             "customer_id": customer_id,
             "billing_address": {
-                "street_address": "123 Abc Way"
+                "street_address": "123 Abc Way",
+                "international_phone": {"country_code": "1", "national_number": "3121234567"}
             }
         })
 
@@ -515,6 +656,8 @@ class TestPaymentMethod(unittest.TestCase):
         found_credit_card = CreditCard.find(token)
         self.assertFalse(found_credit_card is None)
         self.assertTrue(found_credit_card.billing_address.street_address == "123 Abc Way")
+        self.assertEqual("1", found_credit_card.billing_address.international_phone["country_code"])
+        self.assertEqual("3121234567", found_credit_card.billing_address.international_phone["national_number"])
 
     def test_create_overrides_the_billing_address_in_the_nonce(self):
         customer_id = Customer.create().customer.id
@@ -798,7 +941,7 @@ class TestPaymentMethod(unittest.TestCase):
         found_account = PaymentMethod.find(result.payment_method.token)
         self.assertNotEqual(None, found_account)
         self.assertEqual(PayPalAccount, found_account.__class__)
-        self.assertEqual("jane.doe@example.com", found_account.email)
+        self.assertTrue(found_account.email)
 
     def test_find_returns_a_credit_card(self):
         customer = Customer.create().customer
@@ -831,6 +974,35 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertEqual(AndroidPayCard, found_android_pay_card.__class__)
         self.assertEqual(found_android_pay_card.token, android_pay_card.token)
 
+    def test_find_returns_an_apple_pay_mpan_card(self):
+        customer = Customer.create().customer
+        result = PaymentMethod.create({
+            "customer_id": customer.id,
+            "payment_method_nonce": Nonces.ApplePayMpan
+        })
+
+        self.assertTrue(result.is_success)
+        apple_pay_card = result.payment_method
+
+        found_apple_pay_card = PaymentMethod.find(apple_pay_card.token)
+        self.assertIsNotNone(found_apple_pay_card)
+        self.assertEqual(ApplePayCard, found_apple_pay_card.__class__)
+        self.assertEqual(found_apple_pay_card.token, apple_pay_card.token)
+        self.assertEqual(apple_pay_card.merchant_token_identifier, "DNITHE302308980427388297")
+        self.assertEqual(apple_pay_card.source_card_last4, "2006")
+         
+
+    def test_delete_customer_with_path_traversal(self):
+        try:
+            customer = Customer.create({"first_name":"Waldo"}).customer
+            PaymentMethod.delete("../../customers/{}".format(customer.id))
+        except NotFoundError:
+            pass
+
+        found_customer = Customer.find(customer.id)
+        self.assertNotEqual(None, found_customer)
+        self.assertEqual("Waldo", found_customer.first_name)
+
     def test_delete_deletes_a_credit_card(self):
         customer = Customer.create().customer
         result = CreditCard.create({
@@ -880,6 +1052,60 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertTrue(updated_credit_card.last_4 == CreditCardNumbers.MasterCard[-4:])
         self.assertTrue(updated_credit_card.expiration_date == "06/2013")
 
+    def test_update_with_three_d_secure_pass_thru(self):
+        customer_id = Customer.create().customer.id
+        credit_card_result = CreditCard.create({
+            "cardholder_name": "Original Holder",
+            "customer_id": customer_id,
+            "cvv": "123",
+            "number": CreditCardNumbers.Visa,
+            "expiration_date": "05/2012"
+        })
+        update_result = PaymentMethod.update(credit_card_result.credit_card.token, {
+            "cardholder_name": "New Holder",
+            "cvv": "456",
+            "number": CreditCardNumbers.MasterCard,
+            "expiration_date": "06/2013",
+            "three_d_secure_pass_thru": {
+                "three_d_secure_version": "1.1.0",
+                "eci_flag": "05",
+                "cavv": "some-cavv",
+                "xid": "some-xid"
+            },
+            "options": {
+                "verify_card": "true",
+            }
+        })
+
+        self.assertTrue(update_result.is_success)
+
+    def test_create_with_three_d_secure_pass_thru_without_eci_flag(self):
+        customer_id = Customer.create().customer.id
+        credit_card_result = CreditCard.create({
+            "cardholder_name": "Original Holder",
+            "customer_id": customer_id,
+            "cvv": "123",
+            "number": CreditCardNumbers.Visa,
+            "expiration_date": "05/2012"
+        })
+        update_result = PaymentMethod.update(credit_card_result.credit_card.token, {
+            "cardholder_name": "New Holder",
+            "cvv": "456",
+            "number": CreditCardNumbers.MasterCard,
+            "expiration_date": "06/2013",
+            "three_d_secure_pass_thru": {
+                "three_d_secure_version": "1.1.0",
+                "cavv": "some-cavv",
+                "xid": "some-xid"
+            },
+            "options": {
+                "verify_card": "true",
+            }
+        })
+
+        self.assertFalse(update_result.is_success)
+        self.assertEqual("EciFlag is required.", update_result.message)
+
     def test_update_credit_cards_with_account_type_credit(self):
         customer = Customer.create().customer
         result = CreditCard.create({
@@ -987,12 +1213,15 @@ class TestPaymentMethod(unittest.TestCase):
         })
         update_result = PaymentMethod.update(credit_card_result.credit_card.token, {
             "billing_address": {
-                "region": "IL"
+                "region": "IL",
+                "international_phone": {"country_code": "1", "national_number": "3121234567"}
             }
         })
         self.assertTrue(update_result.is_success)
         updated_credit_card = update_result.payment_method
         self.assertTrue(updated_credit_card.billing_address.region == "IL")
+        self.assertEqual("1", updated_credit_card.billing_address.international_phone["country_code"])
+        self.assertEqual("3121234567", updated_credit_card.billing_address.international_phone["national_number"])
         self.assertTrue(updated_credit_card.billing_address.street_address is None)
         self.assertFalse(updated_credit_card.billing_address.id == credit_card_result.credit_card.billing_address.id)
 
@@ -1109,6 +1338,52 @@ class TestPaymentMethod(unittest.TestCase):
         self.assertTrue(update_result.credit_card_verification.status == CreditCardVerification.Status.ProcessorDeclined)
         self.assertTrue(update_result.credit_card_verification.gateway_rejection_reason is None)
 
+    def test_update_includes_risk_data_when_skip_advanced_fraud_checking_is_false(self):
+        with FraudProtectionEnterpriseIntegrationMerchant():
+            customer_id = Customer.create().customer.id
+            credit_card_result = CreditCard.create({
+                "cardholder_name": "Card Holder",
+                "customer_id": customer_id,
+                "cvv": "123",
+                "number": "4111111111111111",
+                "expiration_date": "05/2020"
+            })
+
+            update_result = PaymentMethod.update(credit_card_result.credit_card.token, {
+                "expiration_date": "10/2020",
+                "options": {
+                    "verify_card": True,
+                    "skip_advanced_fraud_checking": False
+                    },
+                })
+
+            self.assertTrue(update_result.is_success)
+            verification = update_result.payment_method.verification
+            self.assertIsInstance(verification.risk_data, RiskData)
+
+    def test_update_does_not_include_risk_data_when_skip_advanced_fraud_checking_is_true(self):
+        with FraudProtectionEnterpriseIntegrationMerchant():
+            customer_id = Customer.create().customer.id
+            credit_card_result = CreditCard.create({
+                "cardholder_name": "Card Holder",
+                "customer_id": customer_id,
+                "cvv": "123",
+                "number": "4111111111111111",
+                "expiration_date": "05/2020"
+            })
+
+            update_result = PaymentMethod.update(credit_card_result.credit_card.token, {
+                "expiration_date": "10/2020",
+                "options": {
+                    "verify_card": True,
+                    "skip_advanced_fraud_checking": True
+                    },
+                })
+
+            self.assertTrue(update_result.is_success)
+            verification = update_result.payment_method.verification
+            self.assertIsNone(verification.risk_data)
+
     def test_update_can_update_the_billing_address(self):
         customer_id = Customer.create().customer.id
         credit_card_result = CreditCard.create({
@@ -1341,46 +1616,26 @@ class TestPaymentMethod(unittest.TestCase):
         granting_gateway, _ = TestHelper.create_payment_method_grant_fixtures()
         self.assertRaises(NotFoundError, granting_gateway.payment_method.revoke, "non-existant-token")
 
-class CreditCardForwardingTest(unittest.TestCase):
-    def setUp(self):
-        braintree.Configuration.configure(
-            braintree.Environment.Development,
-            "forward_payment_method_merchant_id",
-            "forward_payment_method_public_key",
-            "forward_payment_method_private_key"
-        )
+    def test_vault_sepa_direct_debit_payment_method_with_fake_nonce(self):
+        customer_id = Customer.create().customer.id
 
-    def tearDown(self):
-        braintree.Configuration.configure(
-            braintree.Environment.Development,
-            "integration_merchant_id",
-            "integration_public_key",
-            "integration_private_key"
-        )
-
-    def test_forward_raises_exception(self):
-        customer = Customer.create().customer
-        credit_card_result = CreditCard.create({
-            "customer_id": customer.id,
-            "number": "4111111111111111",
-            "expiration_date": "05/2025"
+        result = PaymentMethod.create({
+            "payment_method_nonce": Nonces.SepaDirectDebit,
+            "customer_id": customer_id,
         })
-        self.assertTrue(credit_card_result.is_success)
-        source_merchant_card = credit_card_result.credit_card
 
-        self.assertRaises(NotFoundError, CreditCard.forward, source_merchant_card.token, "integration_merchant_id")
+        self.assertTrue(result.is_success)
 
-    def test_forward_invalid_token_raises_exception(self):
-        self.assertRaises(NotFoundError, CreditCard.forward, "invalid", "integration_merchant_id")
+    def test_delete_sepa_direct_debit_payment_method(self):
+        customer_id = Customer.create().customer.id
 
-    def test_forward_invalid_receiving_merchant_raises_exception(self):
-        customer = Customer.create().customer
-        credit_card_result = CreditCard.create({
-            "customer_id": customer.id,
-            "number": "4111111111111111",
-            "expiration_date": "05/2025"
+        result = PaymentMethod.create({
+            "payment_method_nonce": Nonces.SepaDirectDebit,
+            "customer_id": customer_id,
         })
-        self.assertTrue(credit_card_result.is_success)
-        source_merchant_card = credit_card_result.credit_card
 
-        self.assertRaises(NotFoundError, CreditCard.forward, source_merchant_card.token, "invalid_merchant_id")
+        self.assertTrue(result.is_success)
+
+        delete_result = PaymentMethod.delete(result.payment_method.token)
+
+        self.assertRaises(NotFoundError, PaymentMethod.find, result.payment_method.token)
