@@ -3,7 +3,7 @@ from tests.test_helper import *
 class TestClientToken(unittest.TestCase):
     def test_credit_card_options_require_customer_id(self):
         for option in ["verify_card", "make_default", "fail_on_duplicate_payment_method"]:
-            with self.assertRaisesRegexp(InvalidSignatureError, option):
+            with self.assertRaisesRegex(InvalidSignatureError, option):
                 ClientToken.generate({
                     "options": {option: True}
                 })
