@@ -11,4 +11,4 @@ class TestSignatureService(unittest.TestCase):
     def test_hashes_with_digest(self):
         signature_service = SignatureService("fake_key", FakeDigest.hmac_hash)
         signed = signature_service.sign({"foo": "bar"})
-        self.assertEquals("foo=bar_signed_with_fake_key|foo=bar", signed)
+        self.assertEqual("foo=bar_signed_with_fake_key|foo=bar", signed)
