@@ -6,6 +6,20 @@ from braintree.paypal_account import PayPalAccount
 from braintree.venmo_account import VenmoAccount
 
 class TestWebhooks(unittest.TestCase):
+    def test_granted_payment_method_revoked(self):
+        webhook_testing_gateway = WebhookTestingGateway(BraintreeGateway(Configuration.instantiate()))
+
+        sample_notification = webhook_testing_gateway.sample_notification(WebhookNotification.Kind.GrantedPaymentMethodRevoked, 'granted_payment_method_revoked_id')
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        metadata = notification.revoked_payment_method_metadata
+
+        self.assertEqual(WebhookNotification.Kind.GrantedPaymentMethodRevoked, notification.kind)
+        self.assertEqual("venmo_customer_id", metadata.customer_id)
+        self.assertEqual("granted_payment_method_revoked_id", metadata.token)
+        self.assertEqual(type(metadata.revoked_payment_method), VenmoAccount)
+
     def test_sample_notification_builds_a_parsable_notification(self):
         sample_notification = WebhookTesting.sample_notification(
             WebhookNotification.Kind.SubscriptionWentPastDue,
@@ -30,14 +44,13 @@ class TestWebhooks(unittest.TestCase):
 
         self.assertEqual('my_source_merchant_id', notification.source_merchant_id)
 
-    @raises(InvalidSignatureError)
     def test_completely_invalid_signature(self):
         sample_notification = WebhookTesting.sample_notification(
             WebhookNotification.Kind.SubscriptionWentPastDue,
             "my_id"
         )
-
-        WebhookNotification.parse("bad_stuff", sample_notification['bt_payload'])
+        with self.assertRaises(InvalidSignatureError):
+            WebhookNotification.parse("bad_stuff", sample_notification['bt_payload'])
 
     def test_parse_raises_when_public_key_is_wrong(self):
         sample_notification = WebhookTesting.sample_notification(
@@ -180,6 +193,21 @@ class TestWebhooks(unittest.TestCase):
         self.assertEqual(100, notification.transaction.amount)
         self.assertEqual(datetime(2013, 7, 9, 18, 23, 29), notification.transaction.disbursement_details.disbursement_date)
 
+    def test_builds_notification_for_reviewed_transactions(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.TransactionReviewed,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.TransactionReviewed, notification.kind)
+        self.assertEqual("my_id", notification.transaction_review.transaction_id)
+        self.assertEqual("a smart decision", notification.transaction_review.decision)
+        self.assertEqual("hey@girl.com", notification.transaction_review.reviewer_email)
+        self.assertEqual("I reviewed this", notification.transaction_review.reviewer_note)
+        self.assertEqual(datetime(2021, 4, 20, 6, 9, 0), notification.transaction_review.reviewed_time)
+
     def test_builds_notification_for_settled_transactions(self):
         sample_notification = WebhookTesting.sample_notification(
             WebhookNotification.Kind.TransactionSettled,
@@ -244,6 +272,19 @@ class TestWebhooks(unittest.TestCase):
         self.assertEqual("update_funding_information", notification.disbursement.follow_up_action)
         self.assertEqual(date(2014, 2, 9), notification.disbursement.disbursement_date)
 
+    def test_builds_notification_for_old_dispute_under_review(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeUnderReview,
+            "legacy_dispute_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeUnderReview, notification.kind)
+        self.assertEqual("legacy_dispute_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.UnderReview, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
     def test_builds_notification_for_old_dispute_opened(self):
         sample_notification = WebhookTesting.sample_notification(
             WebhookNotification.Kind.DisputeOpened,
@@ -287,6 +328,71 @@ class TestWebhooks(unittest.TestCase):
         self.assertEqual(notification.dispute.date_opened, date(2014, 3, 28))
         self.assertEqual(notification.dispute.date_won, date(2014, 9, 1))
 
+    def test_builds_notification_for_old_dispute_accepted(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeAccepted,
+            "legacy_dispute_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeAccepted, notification.kind)
+        self.assertEqual("legacy_dispute_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.Accepted, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
+    def test_builds_notification_for_old_dispute_auto_accepted(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeAutoAccepted,
+            "legacy_dispute_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeAutoAccepted, notification.kind)
+        self.assertEqual("legacy_dispute_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.AutoAccepted, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
+    def test_builds_notification_for_old_dispute_disputed(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeDisputed,
+            "legacy_dispute_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeDisputed, notification.kind)
+        self.assertEqual("legacy_dispute_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.Disputed, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
+    def test_builds_notification_for_old_dispute_expired(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeExpired,
+            "legacy_dispute_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeExpired, notification.kind)
+        self.assertEqual("legacy_dispute_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.Expired, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
+    def test_builds_notification_for_new_dispute_under_review(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeUnderReview,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeUnderReview, notification.kind)
+        self.assertEqual("my_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.UnderReview, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
     def test_builds_notification_for_new_dispute_opened(self):
         sample_notification = WebhookTesting.sample_notification(
             WebhookNotification.Kind.DisputeOpened,
@@ -330,6 +436,58 @@ class TestWebhooks(unittest.TestCase):
         self.assertEqual(notification.dispute.date_opened, date(2014, 3, 28))
         self.assertEqual(notification.dispute.date_won, date(2014, 9, 1))
 
+    def test_builds_notification_for_new_dispute_accepted(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeAccepted,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeAccepted, notification.kind)
+        self.assertEqual("my_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.Accepted, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
+    def test_builds_notification_for_new_dispute_auto_accepted(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeAutoAccepted,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeAutoAccepted, notification.kind)
+        self.assertEqual("my_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.AutoAccepted, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
+    def test_builds_notification_for_new_dispute_disputed(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeDisputed,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeDisputed, notification.kind)
+        self.assertEqual("my_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.Disputed, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
+    def test_builds_notification_for_new_dispute_expired(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.DisputeExpired,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.DisputeExpired, notification.kind)
+        self.assertEqual("my_id", notification.dispute.id)
+        self.assertEqual(Dispute.Status.Expired, notification.dispute.status)
+        self.assertEqual(Dispute.Kind.Chargeback, notification.dispute.kind)
+
     def test_builds_notification_for_partner_merchant_connected(self):
         sample_notification = WebhookTesting.sample_notification(
             WebhookNotification.Kind.PartnerMerchantConnected,
@@ -411,6 +569,31 @@ class TestWebhooks(unittest.TestCase):
         self.assertEqual("my_id", notification.connected_merchant_paypal_status_changed.merchant_id)
         self.assertEqual("oauth_application_client_id", notification.connected_merchant_paypal_status_changed.oauth_application_client_id)
 
+    def test_builds_notification_for_refund_failed(self):
+        sample_notification = WebhookTesting.sample_notification(
+                WebhookNotification.Kind.RefundFailed,
+                "my_id"
+         )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.RefundFailed, notification.kind)
+        self.assertEqual("my_id", notification.transaction.id)
+
+    def test_builds_notification_for_subscription_billing_skipped(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.SubscriptionBillingSkipped,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification['bt_signature'], sample_notification['bt_payload'])
+
+        self.assertEqual(WebhookNotification.Kind.SubscriptionBillingSkipped, notification.kind)
+        self.assertEqual("my_id", notification.subscription.id)
+        self.assertTrue(len(notification.subscription.transactions) == 0)
+        self.assertTrue(len(notification.subscription.discounts) == 0)
+        self.assertTrue(len(notification.subscription.add_ons) == 0)
+
     def test_builds_notification_for_subscription_charged_successfully(self):
         sample_notification = WebhookTesting.sample_notification(
             WebhookNotification.Kind.SubscriptionChargedSuccessfully,
@@ -643,3 +826,78 @@ class TestWebhooks(unittest.TestCase):
         self.assertEqual("ee257d98-de40-47e8-96b3-a6954ea7a9a4", local_payment_completed.payment_method_nonce)
         self.assertTrue(isinstance(local_payment_completed.transaction, Transaction))
 
+    def test_local_payment_expired_webhook(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.LocalPaymentExpired,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification["bt_signature"], sample_notification["bt_payload"])
+        local_payment_expired = notification.local_payment_expired
+
+        self.assertEqual(WebhookNotification.Kind.LocalPaymentExpired, notification.kind)
+        self.assertEqual("a-payment-id", local_payment_expired.payment_id)
+        self.assertEqual("a-context-payment-id", local_payment_expired.payment_context_id)
+
+    def test_local_payment_funded_webhook(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.LocalPaymentFunded,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification["bt_signature"], sample_notification["bt_payload"])
+        local_payment_funded = notification.local_payment_funded
+
+        self.assertEqual(WebhookNotification.Kind.LocalPaymentFunded, notification.kind)
+        self.assertEqual("a-payment-id", local_payment_funded.payment_id)
+        self.assertEqual("a-context-payment-id", local_payment_funded.payment_context_id)
+        self.assertTrue(isinstance(local_payment_funded.transaction, Transaction))
+        self.assertEqual("1", local_payment_funded.transaction.id)
+        self.assertEqual("settled", local_payment_funded.transaction.status)
+        self.assertEqual("order1234", local_payment_funded.transaction.order_id)
+
+    def test_local_payment_reversed_webhook(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.LocalPaymentReversed,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification["bt_signature"], sample_notification["bt_payload"])
+        local_payment_reversed = notification.local_payment_reversed
+
+        self.assertEqual(WebhookNotification.Kind.LocalPaymentReversed, notification.kind)
+        self.assertEqual("a-payment-id", local_payment_reversed.payment_id)
+
+    def test_payment_method_customer_data_updated_webhook(self):
+        sample_notification = WebhookTesting.sample_notification(
+            WebhookNotification.Kind.PaymentMethodCustomerDataUpdated,
+            "my_id"
+        )
+
+        notification = WebhookNotification.parse(sample_notification["bt_signature"], sample_notification["bt_payload"])
+        payment_method_customer_data_updated = notification.payment_method_customer_data_updated_metadata
+
+        self.assertEqual(WebhookNotification.Kind.PaymentMethodCustomerDataUpdated, notification.kind)
+
+        self.assertEqual(payment_method_customer_data_updated.token, "TOKEN-12345")
+        self.assertEqual(payment_method_customer_data_updated.datetime_updated, "2022-01-01T21:28:37Z")
+
+        enriched_customer_data = payment_method_customer_data_updated.enriched_customer_data
+        self.assertEqual(enriched_customer_data.fields_updated, ["username"])
+
+        address = {
+            "street_address": "Street Address",
+            "extended_address": "Extended Address",
+            "locality": "Locality",
+            "region": "Region",
+            "postal_code":"Postal Code"
+        }
+
+        profile_data = enriched_customer_data.profile_data
+        self.assertEqual(profile_data.first_name, "John")
+        self.assertEqual(profile_data.last_name, "Doe")
+        self.assertEqual(profile_data.username, "venmo_username")
+        self.assertEqual(profile_data.phone_number, "1231231234")
+        self.assertEqual(profile_data.email, "john.doe@paypal.com")
+        self.assertEqual(profile_data.billing_address, address)
+        self.assertEqual(profile_data.shipping_address, address)
